import dtk_hiv_intrahost as hiv
import random
from collections import defaultdict

class HivPerson():
    def __init__( self, age_in_years=None, sex=None ):
        if sex == None:
            sex = random.randint( 0, 1 )
        elif sex != 0 and sex != 1:
            raise ValueError( "Specified sex needs to be 0 (M) or 1 (F)." )

        if age_in_years == None:
            age_in_years = random.random()*100
        elif age_in_years < 0 or age_in_years > 100:
            raise ValueError( "Specified age needs to be between 0 and 100." )
        self._id = hiv.create( ( sex, age_in_years, 1.0 ) )

    def id( self ):
        """
        Get id of person.
        """
        return self._id

    def sex( self ):
        """
        Get sex of person.
        """
        return self.sex

    def force_infect( self ):
        """
            Initiate an HIV infection.
        """
        hiv.force_infect( self._id )

    def dead( self ):
        """
            Query if person is dead. If not hooked up to demographics module, this can only be from infection.
        """
        return hiv.is_dead( self._id )

    def cd4( self ):
        """
            Query person's CD4 count.
        """
        return( hiv.get_cd4( self._id ) )

    def update( self ):
        """
            Move time forward 1 timestep for this person.
        """
        hiv.update( self._id )

    def serialize( self ):
        """
            Get a json-serialized dictionary representing the full state of this person.
        """
        return hiv.serialize( self._id )

    def age( self ):
        """
            Query person's current age (in days).
        """
        return hiv.get_age( self._id )

    def infected( self ):
        """
            Query person's current HIV-infected status.
        """
        return hiv.is_infected( self._id )

    def infection_age( self ):
        """
            Query person's current HIV-infected status.
        """
        return hiv.get_infection_age( self._id )

    def infectiousness( self ):
        """
            Query person's current per-timestep viral shedding amount.
        """
        return hiv.get_infectiousness( self._id )

    def possible_mother( self ):
        """
            Query whether person can be a mother now.
        """
        return hiv.is_possible_mother( self._id )

    def immunity( self ):
        """
            Query person's immunity.
        """
        return hiv.get_immunity( self._id )

    def pregnant( self ):
        """
            Query person's immunity.
        """
        return hiv.is_pregnant( self._id )
